use utf8;
package Video::PlaybackMachine::Schema;

use strict;
use warnings;

our $VERSION = '0.09'; # VERSION

use base 'DBIx::Class::Schema';

__PACKAGE__->load_namespaces;


# You can replace this text with custom code or comments, and it will be preserved on regeneration
1;
